/************************
 * This class represents a pair of a String and a signed int.  Allows convenient
 * storage into another data type.
 * @author Jefferson Ng
 * @see ElevenClient*/

import java.util.*;
import java.io.Serializable;  //so enclosing data structures may be tossed
                              //through an ObjectOutputStream


public class ElevenDBData implements Serializable {
  private String username;
  private int value;
  private String nick;
  private String pwd;
  private String quote;
  private String trades;
  private String email;
  private String screen;
  private LinkedList history;
  private Date lastLogin;
  private LinkedList buddyList;
  private LinkedList messages;

  private static final String DELIM = ElevenDB.DELIM;

  /**Create an object with the given user and value*/
  public ElevenDBData(String user, int value) {
    this(user, value, "DEFAULT_NICK", "DEFAULT_PASS");
  }

  /**Create a data spot with nick and password as well*/
  public ElevenDBData(String user, int value, String nick, String pwd) {
    this(user, value, nick, pwd, "NO_EMAIL", "NO_TRADES", "NO_EMAIL", "NO_SCREEN",
	 new LinkedList());
  }
  /**Create all fields except LinkedList*/
  public ElevenDBData(String user, int value, String nick, String pwd, 
		      String quote, String trades, String email, String screen) {
    this(user, value, nick, pwd, quote, trades, email, screen, new LinkedList());
  }

  /**Create all fields except IM buddylist and message list*/
  public ElevenDBData(String user, int value, String nick, String pwd, 
		      String quote, String trades, String email, String screen,
		      LinkedList hist) {
    this(user, value, nick, pwd, quote, trades, email, screen, new LinkedList(),
	 new LinkedList(), new LinkedList());
  }

  /**Create all fields*/
  public ElevenDBData(String user, int value, String nick, String pwd, 
		      String quote, String trades, String email, String screen,
		      LinkedList hist, LinkedList buddyList, LinkedList messages) {

    this.username = user;
    this.value = value;
    this.nick = nick;
    this.pwd = pwd;
    this.quote = quote;
    this.trades = trades;
    this.email = email;
    this.screen = screen;
    history = hist;
    lastLogin = null;
    this.buddyList = buddyList;
    this.messages = messages;
  }

  //Accessors, Mutators
  public String getUser() { return username; }
  public int getValue() { return value; }
  public String getNick() { return nick; }
  public String getPwd() { return pwd; }
  public String getQuote() { return quote; }
  public String getTrades() { return trades; }
  public String getEmail() { return email; }
  public String getScreen() { return screen; }
  public LinkedList getHistory() { return history; }
  public ListIterator getIterator() { return getHistory().listIterator(); }
  public Date getLastLogin() { return lastLogin; }
  public LinkedList getBuddyList() { return buddyList; }
  public LinkedList getMessages() { return messages; }


  public void setUser(String user) { this.username = user; }
  public void setValue(int value) { this.value = value; }
  public void setNick(String nick) { this.nick = nick; }
  public void setPwd(String pwd) { this.pwd = pwd; }
  public void setQuote(String quote) { this.quote = quote; }
  public void setTrades(String trades) { this.trades = trades; }
  public void setEmail(String email) { this.email = email; }
  public void setScreen(String screen) { this.screen = screen; }
  public void setHistory(LinkedList history) { this.history = history; }
  public void updateLastLogin() { this.lastLogin = Calendar.getInstance().getTime(); }
  public void setBuddylist(LinkedList buddyList) { this.buddyList = buddyList; }
  public void setMessages(LinkedList messages) { this.messages = messages; }

  public String toString() { 
    return username +DELIM + value + DELIM + nick + DELIM + pwd + DELIM + 
      quote + DELIM + trades + DELIM + email + DELIM + screen + DELIM + lastLogin;
  }

  //returns the name and password
  public String netStringInfo() { return nick + DELIM + pwd; }
  //returns the name and password
  public String netStringInfoHidden() { return nick + DELIM + "*SECRET*"; }

  //returns the complete details the server would return
  public String netStringDetails() { return quote + DELIM + trades + DELIM +
				       email + DELIM + screen; }
  //returns a semi-hidden set of info.  For general queries without a password
  public String netStringDetailsHidden() { return quote + DELIM + trades + DELIM +
					     "*PRIVATE*" + DELIM + "*PRIVATE*"; }

  public String stringHistory() {
    ListIterator l = history.listIterator();
    String ans = "";
    while(l.hasNext()) {
      ans += (ElevenDBTransaction) l.next();
    }
    return ans;
  }
}
